/*
Firmware for a wireless RGB LED-bulb implemented in an ATTiny44.
(C) 2009 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "io.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>

unsigned int setCurrent[3];

void setCurrentFor(char chan, unsigned int val) {
	if (val==0) {
		if (chan==CHAN_RED) TCCR0A&=0x3F;
		if (chan==CHAN_GREEN) TCCR0A&=0xCF;
		if (chan==CHAN_BLUE) TCCR1A&=0x3F;
	} else if (setCurrent[chan]==0) {
		if (chan==CHAN_RED) TCCR0A|=0x80;
		if (chan==CHAN_GREEN) TCCR0A|=0x20;
		if (chan==CHAN_BLUE) TCCR1A|=0x80;
	}
	setCurrent[chan]=val;
}

inline void rgbSetChan(char chan, unsigned char val) {
	if (chan==CHAN_RED) OCR0A=val;
	if (chan==CHAN_GREEN) OCR0B=val;
	if (chan==CHAN_BLUE) OCR1A=val;
}

int getCurrent(char chan) {
	ADCSRA=0; //disable ADC
	if (chan==CHAN_RED) ADMUX=0x0|0x1B;
	if (chan==CHAN_GREEN) ADMUX=0x0|0x11;
	if (chan==CHAN_BLUE) ADMUX=0x0|0x09;
	ADCSRA=0xC7; //start conv, presc = /128
	while(ADCSRA&0x40) ;
	return ADC;
}

//Debugging: Store a pwm<->current-curve for a given channel into the EEPROM.
static void makeCurrentCurve(char chan) {
	int addr, val, n;

	//Wait a while
	for (int n=0; n<4000; n++) {
		_delay_ms(1);
	}
	
	//Erase eeprom
	for (addr=0; addr<256; addr++) {
		eeprom_write_byte(addr, 0xff);
	}
	
	//Go measure current
	addr=0;
	do {
		rgbSetChan(chan, addr);
		val=0;
		for (n=0; n<64; n++) {
			_delay_ms(1);
			val+=getCurrent(chan);
		}
		eeprom_write_byte(addr++, val>>8);
		eeprom_write_byte(addr++, val&0xff);
	} while (addr<256); // && val<0x3f0);
	//All done, kill led.
	rgbSetChan(chan, 0);
	//and halt
	while(1);
}

void ioInit(void) {
	DDRA=0xC0;
	DDRB=0x6;
	TCCR0A=0xA3; //fast PWM on ocr0a, ocr0b
	TCCR0B=0x1; //no prescaling
	TCCR1A=0x81; //fast PWM, 8bit, on ocr1a
	TCCR1B=0x9; //no prescaling
	
	DIDR0=0x3f; //disable digital in buff of ad chans
	//turn off leds
	OCR0A=0;
	OCR0B=0;
	OCR1A=0;

	//Fire off initial (bogus) AD conversion
	ADCSRA=0xc8;
//	makeCurrentCurve(CHAN_RED);
}


#define SAMP_COUNT 16
#define SAMP_TRESH_LO 2
#define SAMP_TRESH_HI (SAMP_COUNT-2)

ISR(ADC_vect, ISR_NOBLOCK) {
	static char sampleNo=0;
	static char samplesHigher[3];

	int val;
	val=ADC;
	if (ADMUX==MUX_CHAN_RED) {
		if (val>setCurrent[0]) samplesHigher[CHAN_RED]++;
		ADMUX=MUX_CHAN_GREEN;
	} else if (ADMUX==MUX_CHAN_GREEN) {
		if (val>setCurrent[1]) samplesHigher[CHAN_GREEN]++;
		ADMUX=MUX_CHAN_BLUE;
	} else if (ADMUX==0xA2) { //temp sens
		if (val>350) { //temp > 80 deg C?
			//Go to dim red only to indicate over-temp
			setCurrentFor(0,20);
			setCurrentFor(1,0);
			setCurrentFor(2,0);
		}
		ADMUX=MUX_CHAN_RED;
	} else { //CHAN_BLUE
		if (val>setCurrent[2]) samplesHigher[CHAN_BLUE]++;
		ADMUX=MUX_CHAN_RED;
		sampleNo++;
	} 

	if (sampleNo==SAMP_COUNT) {
		if (samplesHigher[CHAN_RED]<SAMP_TRESH_LO) OCR0A++;
		if (samplesHigher[CHAN_RED]>SAMP_TRESH_HI) OCR0A--;
		if (samplesHigher[CHAN_GREEN]<SAMP_TRESH_LO) OCR0B++;
		if (samplesHigher[CHAN_GREEN]>SAMP_TRESH_HI) OCR0B--;
		if (samplesHigher[CHAN_BLUE]<SAMP_TRESH_LO) OCR1A++;
		if (samplesHigher[CHAN_BLUE]>SAMP_TRESH_HI) OCR1A--;
		samplesHigher[0]=0;
		samplesHigher[1]=0;
		samplesHigher[2]=0;
		sampleNo=0;
		if (OCR0A==0xFF) OCR0A=0;
		if (OCR0B==0xFF) OCR0B=0;
		if (OCR1A==0xFF) OCR1A=0;
		if (OCR0A==0xF1) OCR0A=0xF0;
		if (OCR0B==0xF1) OCR0B=0xF0;
		if (OCR1A==0xF1) OCR1A=0xF0;
		//Next try, get temp.
		ADMUX=0xA2;
	}
	
	ADCSRA=0;
	ADCSRA=0xCB; //start conv, presc = /128, int on
//	ADCSRA=0xCF; //start conv, presc = /128, int on
}