/*
Firmware for a wireless RGB LED-bulb implemented in an ATTiny44.
(C) 2009 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "io.h"
#include <avr/io.h>
#include <avr/interrupt.h>

#define TIMER_TICK_US 32
#define RF_LONG (1300/TIMER_TICK_US)
#define RF_SHORT (640/TIMER_TICK_US)
#define MARGIN (500/TIMER_TICK_US)

/*
Protocol:
Long=1300uS, short=620uS.

- Start: high for a 'short' period
- 20 bits:
  0? low, wait 'short', high, wait 'long'
  1? low, wait 'long', high, wait 'short'
- End: low for 80mS

The 20 bits we receive and accept have the following value:
0xVVA5T

With VV being the 8-bit value, A5 the magic (=0xA5) to indicate we want to 
talk to this unit, and T the channel: 0=red, 1=green, 2=blue.

*/

static unsigned int rfTimer;
char rfState=-1;

void rfInit(void) {
	//RF module is connected to PB0/PCINT8
	PCMSK1=1; //enable pin change int on PCINT8
	PCMSK0=0;
	GIMSK|=0x20; //enable pcint11..8
	//We need something to keep the time too. Enable timer0 ovf to that end.
	TIMSK0|=1;
}

ISR(TIM0_OVF_vect) {
	//rfTimer will be increased with a frequency of (8000000/256)=31250Hz
	//which is exactly every 32uS.
	rfTimer++;
//	if (rfTimer>10) GIMSK|=0x20; //enable pcint11..8
	if (rfTimer>(2000/TIMER_TICK_US)) rfState=-1;
}

#define RFPIN_LOW (!(PINB&(1<<0)))
#define RFPIN_HIGH (PINB&(1<<0))
#define IS_SHORT ((rfTimer<(RF_SHORT+MARGIN)) && (rfTimer>(RF_SHORT-MARGIN)))
#define IS_LONG ((rfTimer<(RF_LONG+MARGIN)) && (rfTimer>(RF_LONG-MARGIN)))

ISR(PCINT1_vect) {
	static unsigned long val;
//	GIMSK&=~0x20;
	
	PORTB|=2;
	if (rfState==-1 && RFPIN_HIGH) {
		//Perhaps a start event.
		rfState=0;
		val=0;
	} else if (rfState==0 && RFPIN_LOW && IS_SHORT) {
		//End of start event.
		rfState=1;
	} else if ((rfState>=1) && (rfState<=20)) {
		if (RFPIN_HIGH && (IS_SHORT || IS_LONG)) {
			//meh
		} else if (RFPIN_LOW && (IS_SHORT || IS_LONG)) {
			if (IS_SHORT) val|=(1l<<20l);
			val>>=1;
			rfState++;
		} else {
			rfState=-1; //not recognized
		}
		if (rfState==21) {
			int c=(val>>12);
			c<<=2;
			if ((val&0xFFFl)==0xA50) setCurrentFor(CHAN_RED, c);
			if ((val&0xFFFl)==0xA51) setCurrentFor(CHAN_GREEN, c);
			if ((val&0xFFFl)==0xA52) setCurrentFor(CHAN_BLUE, c);
			rfState=-1;
		}
	} else {
		rfState=-1; //ignore :/
		if (RFPIN_HIGH) {
			rfState=0;
			val=0;
		}
	}
	
	if (rfState==-1) PORTB&=~2;
	
	rfTimer=0;
}